"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseConfig = parseConfig;
exports.mapProps = mapProps;

var _upperFirst2 = _interopRequireDefault(require("lodash/upperFirst"));

var _isNaN2 = _interopRequireDefault(require("lodash/isNaN"));

var _dom = _interopRequireDefault(require("sketch/dom"));

var _settings = _interopRequireDefault(require("sketch/settings"));

var _data = require("./utils/data");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const Flow = _dom.default.Flow;
const commonProps = {
  id: true,
  name: true
};
const availableProps = {
  artboard: _objectSpread({}, commonProps, {
    from: true
  }),
  group: _objectSpread({}, commonProps, {
    flow: true
  }),
  text: _objectSpread({}, commonProps, {
    flow: true,
    text: true,
    alignment: true
  }),
  shape: _objectSpread({}, commonProps, {
    flow: true
  }),
  image: _objectSpread({}, commonProps, {
    flow: true,
    image: true
  }),
  symbolmaster: _objectSpread({}, commonProps, {
    flow: true,
    from: true,
    symbolId: true
  }),
  symbolinstance: _objectSpread({}, commonProps, {
    flow: true,
    from: true
  })
};
const specialProps = {
  flow(props) {
    if (props.flow && !!props.flow.target || !!props.flow.targetId) {
      const _props$flow = props.flow,
            target = _props$flow.target,
            targetId = _props$flow.targetId,
            _props$flow$animation = _props$flow.animationType,
            animationType = _props$flow$animation === void 0 ? String(Flow.AnimationType.slideFromLeft) : _props$flow$animation;
      const flow = {};

      if (target) {
        flow.target = target;
      } else {
        flow.targetId = targetId;
      }

      flow.animationType = String(animationType);
      return flow;
    }

    return null;
  }

};

function parseConfig(type, props) {
  const config = {};
  const availables = availableProps[type];
  Object.entries(props).forEach(([propName, propValue]) => {
    if (typeof specialProps[propName] === 'function') {
      if (specialProps[propName](props)) {
        config[propName] = specialProps[propName](props);
      }
    } else if (availables[propName]) {
      config[propName] = propValue;
    }
  });
  return config;
}

const mapPropsStrategy = {
  systemFontSize(layer, fontSize) {
    if (!(0, _isNaN2.default)(Number(fontSize)) && layer.type === String(_dom.default.Types.Text)) {
      layer.sketchObject.setFontSize(Number(fontSize));
    }
  },

  data(layer, data) {
    if (typeof _settings.default.setLayerSettingForKey === 'function') {
      Object.entries(data).forEach(([key, value]) => {
        _settings.default.setLayerSettingForKey(layer, key, value);
      });
    } else {
      Object.entries(data).forEach(([key, value]) => {
        (0, _data.setLayerSettingForKey)(layer, key, value);
      });
    }
  },

  overrides(instance, overrides) {
    if (instance.type === String(_dom.default.Types.SymbolInstance)) {
      if (Array.isArray(overrides)) {
        instance.overrides.forEach((override, index) => {
          const value = overrides[index];

          if (value !== undefined) {
            override.value = value;
          }
        });
      } else if (typeof overrides === 'object' && overrides !== null) {
        instance.overrides.forEach(override => {
          const name = String(override.sketchObject.overridePoint().layerName());
          const value = overrides[name];

          if (value !== undefined) {
            override.value = value;
          }
        });
      }
    }
  },

  lock(layer, isLocked) {
    layer.sketchObject.setIsLocked(!!isLocked);
  },

  fix(layer, fixProps) {
    if (Array.isArray(fixProps)) {
      fixProps.forEach(prop => {
        layer.sketchObject[`hasFixed${(0, _upperFirst2.default)(prop)}`] = true;
      });
    } else if (typeof fixProps === 'object' && fixProps !== null) {
      Object.entries(fixProps).forEach(([prop, value]) => {
        layer.sketchObject[`hasFixed${(0, _upperFirst2.default)(prop)}`] = !!value;
      });
    }
  }

};

function mapProps(layer, props) {
  Object.entries(props).forEach(([propName, propValue]) => {
    if (typeof mapPropsStrategy[propName] === 'function') {
      mapPropsStrategy[propName](layer, propValue);
    }
  });
}