"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mapStyles = mapStyles;

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _isNaN2 = _interopRequireDefault(require("lodash/isNaN"));

var _dom = _interopRequireDefault(require("sketch/dom"));

var _render = require("./render");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const mapStylesStrategy = {
  backgroundColor(layer, color, fill) {
    fill.color = _dom.default.Style.colorFromString(color);
  },

  borderColor(layer, color, border) {
    border.color = _dom.default.Style.colorFromString(color);
  },

  borderWidth(layer, width, border) {
    width = parseInt(width, 10);

    if (!(0, _isNaN2.default)(width)) {
      border.thickness = width;
    }
  },

  borderRadius(layer, radius, __, shape) {
    if (!shape.layers()[0]) {
      return;
    }

    shape.layers()[0].setCornerRadiusFromComponents(`${radius};${radius};${radius};${radius}`);
  },

  shadowColor(layer, color, shadow) {
    shadow.setColor(_dom.default.Style.colorFromString(color));
  },

  shadowX(layer, offsetX, shadow) {
    offsetX = parseInt(offsetX, 10);

    if (!(0, _isNaN2.default)(offsetX)) {
      shadow.offsetX = offsetX;
    }
  },

  shadowY(layer, offsetY, shadow) {
    offsetY = parseInt(offsetY, 10);

    if (!(0, _isNaN2.default)(offsetY)) {
      shadow.offsetY = offsetY;
    }
  },

  shadowBlur(layer, blur, shadow) {
    blur = parseInt(blur, 10);

    if (!(0, _isNaN2.default)(blur)) {
      shadow.blurRadius = blur;
    }
  },

  shadowSpread(layer, spread, shadow) {
    spread = parseInt(spread, 10);

    if (!(0, _isNaN2.default)(spread)) {
      shadow.spread = spread;
    }
  }

};

function mapStyles(layer, style) {
  // TODO: render style to native shape
  if ((0, _isEmpty2.default)(style) || !_render.canContainsChildren[layer.type.toLowerCase()]) return;
  const frame = new _dom.default.Rectangle(0, 0, layer.frame.width, layer.frame.height);
  const shape = new _dom.default.Shape({
    frame
  }).sketchObject;
  const fill = shape.style().addStylePartOfType(0);
  fill.color = _dom.default.Style.colorFromString('rgba(0, 0, 0, 0)');
  const border = shape.style().addStylePartOfType(1);
  border.color = _dom.default.Style.colorFromString('rgba(0, 0, 0, 0)');
  const shadow = shape.style().addStylePartOfType(2);
  shadow.color = _dom.default.Style.colorFromString('rgba(0, 0, 0, 0)');
  Object.entries(style).forEach(([styleName, styleValue]) => {
    if (typeof mapStylesStrategy[styleName] === 'function') {
      let shapeStyle;

      if (/^background/.test(styleName)) {
        shapeStyle = fill;
      } else if (/^border/.test(styleName)) {
        shapeStyle = border;
      } else if (/^shadow/.test(styleName)) {
        shapeStyle = shadow;
      }

      mapStylesStrategy[styleName](layer, styleValue, shapeStyle, shape);
    } else {
      // eslint-disable-next-line no-console
      console.warn(`unknown property ${style}`);
    }
  });
  layer.sketchObject.addLayer(shape);
  MSLayerMovement.moveToBack([shape]);
}